# Sharpness-aware Quantization for Deep Neural Networks

## Setup the environments

Install pytorch 1.8+, tensorboard and prettytable

```bash
conda install pytorch torchvision torchaudio cudatoolkit=11.3 -c pytorch
pip install tensorboard
pip install prettytable
```

## Data preparation

### ImageNet

Download the ImageNet 2012 dataset from [here](http://image-net.org/), and prepare the dataset based on this [script](https://gist.github.com/BIGBALLON/8a71d225eff18d88e469e6ea9b39cef4). 

## Training

### Train full-precision models

1. Before training, you need to set `data_path` in `configs/full_preicision/imagenet_resnet.json`.

```bash
vim configs/full_preicision/imagenet_resnet.json
```

2. Train full-precision ResNet-18 on ImageNet.

```bash
sh script/full_precision/train_imagenet_resnet.sh
```

### Train low-precision models with SGD

1. Before training, you need to set `data_path` and `pretrained` in `configs/quantization/sgd/imagenet_resnet_w2a2.json`. You can also to specify the bitwidths of weights and activations by `qw` and `qa` .

```bash
vim configs/quantization/sgd/imagenet_resnet_w2a2.json
```

2. Train low-precision ResNet-18 with SGD on ImageNet.

```bash
sh script/quantization/sgd/train_imagenet_resnet_w2a2.sh
```

### Train low-precision models with SAQ

1. Before training, you need to set `data_path` and `pretrained` in `configs/quantization/saq/imagenet_resnet_w2a2.json`. You can also to specify the bitwidths of weights and activations by `qw` and `qa` .

```bash
vim configs/quantization/saq/imagenet_resnet_w2a2.json
```

2. Train low-precision ResNet-18 with SAQ on ImageNet.

```bash
sh script/quantization/saq/train_imagenet_resnet_w2a2.sh
```

## Results of CNNs on ImageNet
|  Network  | Method | Bitwidth (W/A) | Top-1 Acc. (%) | Top-5 Acc. (%) |
| :-------: | :----: | :------------: | :------------: | :------------: |
| ResNet-18 |  SGD   |      2/2       |      66.8      |      87.3      |
| ResNet-18 |  SAQ   |      2/2       |    **67.5**    |    **87.5**    |
| ResNet-18 |  SGD   |      4/4       |      71.1      |      89.8      |
| ResNet-18 |  SAQ   |      4/4       |    **71.6**    |    **90.1**    |
| ResNet-50 |  SGD   |      2/2       |      73.9      |      91.6      |
| ResNet-50 |  SGD   |      2/2       |    **74.5**    |    **91.9**    |
| ResNet-50 |  SGD   |      4/4       |      76.5      |      93.1      |
| ResNet-50 |  SGD   |      4/4       |    **77.6**    |    **93.6**    |
