import json
import os
import sys

from core.config import create_dir, get_parser, params_check


def get_qparser():
    parser = get_parser()
    # general
    parser.add_argument(
        "--quantize_first_last",
        type=bool,
        default=True,
        help="whether to quantize the first and last layer",
    )
    parser.add_argument(
        "--clip_lr", type=float, default=0.01, help="learning rate of clip value"
    )
    parser.add_argument("--qw", type=float, default=8.0, help="weight bit")
    parser.add_argument("--qa", type=float, default=8.0, help="activation bit")
    parser.add_argument("--rho", type=float, default=0.1, help="rho in SAQ")
    parser.add_argument(
        "--include_wclip",
        type=bool,
        default=False,
        help="whether to include clip of weight in SAQ",
    )
    parser.add_argument(
        "--include_aclip",
        type=bool,
        default=False,
        help="whether to include clip of activation in SAQ",
    )
    parser.add_argument(
        "--include_bn",
        type=bool,
        default=True,
        help="whether to include bn parameters in SAQ",
    )
    parser.add_argument(
        "--train_ratio", type=float, default=0.5, help="ratio of training set"
    )
    parser.add_argument(
        "--val_num", type=int, default=10000, help="number of validation samples"
    )

    parser.add_argument("--config", default=None, type=str, help="model configuration")
    return parser


def set_save_path(args):
    if len(args.suffix) == 0:
        suffix = "log_{}_{}_bs{:d}_e{:d}_lr{:.5f}_cliplr{}_w{}a{}_qfl{}_opt{}_rho{}_iwc{}_iac{}_ib{}_{}_{}/".format(
            args.network,
            args.dataset,
            args.batch_size,
            args.n_epochs,
            args.lr,
            args.clip_lr,
            args.qw,
            args.qa,
            1 if args.quantize_first_last else 0,
            args.opt_type,
            args.rho,
            args.include_wclip,
            args.include_aclip,
            args.include_bn,
            args.date,
            args.experiment_id,
        )
    else:
        suffix = args.suffix

    args.save_path = os.path.join(args.save_path, suffix)

    create_dir(args.save_path)


parser = get_qparser()
args = parser.parse_args()
if args.config is not None:
    config_args = json.load(open(args.config))
    override_keys = {
        arg[2:].split("=")[0] for arg in sys.argv[1:] if arg.startswith("--")
    }
    for k, v in config_args.items():
        if k not in override_keys:
            setattr(args, k, v)
del args.config
params_check(args)
