import json
import os
import sys

from core.config import create_dir, get_parser, params_check


def get_qparser():
    parser = get_parser()
    # general
    parser.add_argument(
        "--quantize_first_last",
        type=bool,
        default=True,
        help="whether to quantize the first and last layer",
    )
    parser.add_argument(
        "--clip_lr", type=float, default=0.01, help="learning rate of clip value"
    )
    parser.add_argument("--qw", type=float, default=8.0, help="weight bit")
    parser.add_argument("--qa", type=float, default=8.0, help="activation bit")
    parser.add_argument("--eval", action="store_true", help="Perform evaluation only")
    parser.add_argument("--config", default=None, type=str, help="model configuration")
    return parser


def set_save_path(args):
    if len(args.suffix) == 0:
        suffix = "log_{}_{}_bs{:d}_e{:d}_lr{:.5f}_clr{}_w{}a{}_qfl{}_{}_{}/".format(
            args.network,
            args.dataset,
            args.batch_size,
            args.n_epochs,
            args.lr,
            args.clip_lr,
            args.qw,
            args.qa,
            1 if args.quantize_first_last else 0,
            args.date,
            args.experiment_id,
        )
    else:
        suffix = args.suffix

    args.save_path = os.path.join(args.save_path, suffix)

    create_dir(args.save_path)


def get_args():
    parser = get_qparser()
    args = parser.parse_args()
    if args.config is not None:
        config_args = json.load(open(args.config))
        override_keys = {
            arg[2:].split("=")[0] for arg in sys.argv[1:] if arg.startswith("--")
        }
        for k, v in config_args.items():
            if k not in override_keys:
                setattr(args, k, v)
    del args.config
    params_check(args)
    return args
